==> ldapsaisie_1.4-2.tar.gz: ldapsaisie_clean/public_html/includes/class/class.LSformElement_jsonCompositeAttribute.php <==
<?php
/*******************************************************************************
 * Copyright (C) 2007 Easter-eggs
 * http://ldapsaisie.labs.libre-entreprise.org
 *
 * Author: See AUTHORS file in top-level directory.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

LSsession :: loadLSclass('LSformElement');

/**
 * Element jsonCompositeAttribute for LSform
 *
 * This classe permit to handle compostie attributes encoded with JSON.
 *
 * @author Benjamin Renard <brenard@easter-eggs.com>
 */

class LSformElement_jsonCompositeAttribute extends LSformElement {

  var $template = 'LSformElement_jsonCompositeAttribute.tpl';
  var $fieldTemplate = 'LSformElement_jsonCompositeAttribute_field.tpl';

  function LSformElement_jsonCompositeAttribute (&$form, $name, $label, $params,&$attr_html){
    parent :: LSformElement($form, $name, $label, $params,$attr_html);
    if (is_array($this -> params['html_options']['components'])) {
      $this -> components = $this -> params['html_options']['components'];
    }
  }

  /*
   * Value components :
   *
   * Format :
   *   array (
   *     '[component1_key]' => array (
   *       'label' => '[component label]',
   *       'type' => '[component type]',
   *       'required' => '[booléen]',
   *       'check_data' =>  array([config LSform_rule])
   *     ),
   *     '[component2_key]' => array (
   *       'label' => 'label2',
   *       'type'  => 'select_list',
   *       'options' => array([config as LSattr_html_select_list html_options]),
   *     ),
   *     [...]
   *   )
   * Types :
   *   - 'select_list' => Component feed by a list of valeur configured like an
   *                      atribute LSattr_html :: select_list.
   *   - 'text'        => manual entry
   *
   */
  var $components = array();

 /**
  * Retourne les infos d'affichage de l'élément
  *
  * Cette méthode retourne les informations d'affichage de l'élement
  *
  * @retval array
  */
  function getDisplay(){
    $return = $this -> getLabelInfos();

    $parseValues=array();
    $invalidValues=array();
    foreach($this -> values as $val) {
      $decodedValue=json_decode($val, true);
      if (is_array($decodedValue)) {
        $parseValue=array('value' => $val);
        foreach($decodedValue as $c => $cvalue) {
          $parseValue[$c]=$this -> translateComponentValue($c,$cvalue);
        }
        $parseValues[]=$parseValue;
      }
      else {
        $invalidValues[]=$val;
      }
    }

    $components = $this -> components;
    foreach($components as $c => $cconf) {
      if ($cconf['type']=='select_list') {
        $components[$c]['possible_values']=$this -> getSelectListComponentPossibleValues($c);
      }
    }

    $return['html'] = $this -> fetchTemplate(NULL,
      array(
        'parseValues' => $parseValues,
        'components' => $components
      )
    );
    LSsession :: addCssFile('LSformElement_jsonCompositeAttribute.css');
    return $return;
  }


 /**
  * Return HTML code of an empty field
  *
  * @retval string HTML code of an empty field.
  */
  function getEmptyField() {
    return $this -> fetchTemplate($this -> fieldTemplate,array('components' => $this -> components));
  }

  /**
   * Translate componant value
   *
   * Return an array containing :
   *  - value : untranslated value
   *  - translated : translated value
   *
   * @param[in] $c string The component name
   * @param[in] $value string The value
   *
   * @retval array
   **/
  function translateComponentValue($c,$value) {
    $retval = array (
      'translated' => $value,
      'value' => $value,
    );
    if (isset($this -> components[$c])) {
      if ($this -> components[$c]['type']=='select_list') {
        $retval['translated'] = $this -> getSelectListComponentValueLabel($c,$value);
      }
      //elseif type == 'text' => no transformation
    }
    return $retval;
  }

  /**
   * Retreive possible values of an select_list component
   *
   * @param[in] $c string The component name
   *
   * @retval array
   **/
  protected $_cache_getSelectListComponentPossibleValues=array();
  protected function getSelectListComponentPossibleValues($c) {
    if (!isset($this -> _cache_getSelectListComponentPossibleValues[$c])) {
      if (!LSsession :: loadLSclass('LSattr_html_select_list')) return;
      $this -> _cache_getSelectListComponentPossibleValues[$c]=LSattr_html_select_list :: getPossibleValues($this -> components[$c]['options'], $this -> name, $this->attr_html->attribute->ldapObject);
    }
    return $this -> _cache_getSelectListComponentPossibleValues[$c];
  }

  /**
   * Retreive value's label of an select_list component
   *
   * @param[in] $c string The component name
   * @param[in] $value string The value
   *
   * @retval array
   **/
  protected function getSelectListComponentValueLabel($c,$value) {
    if ($this -> getSelectListComponentPossibleValues($c)) {
      foreach ($this -> _cache_getSelectListComponentPossibleValues[$c] as $v => $label) {
        if (is_array($label)) {
          if (!isset($label['possible_values'])) continue;
          foreach ($label['possible_values'] as $vk => $vl)
            if ($vk == $$value) return $vl;
        }
        if ($v == $value) return $label;
      }
    }
    return;
  }

  /**
   * Retreive LSformElement value from POST data
   *
   * This method check present of this element's value in POST data and retreive
   * it to feed the array passed in paramater.
   *
   * @param[] array Reference of the array for retreived values
   *
   * @retval boolean true if value is in POST data, false instead
   */
  function getPostData(&$return) {
    if($this -> isFreeze()) {
      return true;
    }

    $count=0;
    $end=false;
    $return[$this -> name]=array();
    while ($end==false) {
      $value=array();
      $parseValue=array();
      $errors=array();
      $unemptyComponents=array();
      foreach ($this -> components as $c => $cconf) {
        if (isset($_POST[$this -> name.'__'.$c][$count])) {
          $parseValue[$c]=$_POST[$this -> name.'__'.$c][$count];
          if ($cconf['required'] && empty($parseValue[$c])) {
            $errors[]=getFData(__('Component %{c} must be defined'),__($cconf['label']));
            continue;
          }
          if (empty($parseValue[$c])) {
            continue;
          }
          $unemptyComponents[]=$c;
          if ($cconf['type']=='select_list') {
            if (!$this -> getSelectListComponentValueLabel($c, $parseValue[$c])) {
              $errors[]=getFData(__('Invalid value for component %{c}.'),__($cconf['label']));
            }
          }
          if (is_array($cconf['check_data'])) {
            foreach($cconf['check_data'] as $ruleType => $rconf) {
              $className='LSformRule_'.$ruleType;
              if (LSsession::loadLSclass($className)) {
                $r=new $className();
                if (!$r -> validate($parseValue[$c],$rconf,$this)) {
                  if (isset($rconf['msg'])) {
                    $errors[]=getFData(__($rconf['msg']),__($cconf['label']));
                  }
                  else {
                    $errors[]=getFData(__('Invalid value for component %{c}.'),__($cconf['label']));
                  }
                }
              }
              else {
                $errors[]=getFData(__("Can't validate value of component %{c}."),__($cconf['label']));
              }
            }
          }
          $value[$c]=$parseValue[$c];
        }
        else {
          // end of value break
          $end=true;
          break;
        }

      }
      if (!$end) {
        if (!empty($unemptyComponents)) {
          foreach($errors as $e) {
            $this -> form -> setElementError($this -> attr_html,$e);
          }
          $return[$this -> name][]=json_encode($value);
        }
        $count++;
      }
    }
    return true;
  }

}

==> ldapsaisie_1.7-1.tar.gz: ldapsaisie_clean/public_html/includes/class/class.LSformElement_jsonCompositeAttribute.php <==
<?php
/*******************************************************************************
 * Copyright (C) 2007 Easter-eggs
 * http://ldapsaisie.labs.libre-entreprise.org
 *
 * Author: See AUTHORS file in top-level directory.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

LSsession :: loadLSclass('LSformElement');

/**
 * Element jsonCompositeAttribute for LSform
 *
 * This classe permit to handle compostie attributes encoded with JSON.
 *
 * @author Benjamin Renard <brenard@easter-eggs.com>
 */

class LSformElement_jsonCompositeAttribute extends LSformElement {

  var $template = 'LSformElement_jsonCompositeAttribute.tpl';
  var $fieldTemplate = 'LSformElement_jsonCompositeAttribute_field.tpl';

  function LSformElement_jsonCompositeAttribute (&$form, $name, $label, $params,&$attr_html){
    parent :: LSformElement($form, $name, $label, $params,$attr_html);
    if (is_array($this -> params['html_options']['components'])) {
      $this -> components = $this -> params['html_options']['components'];
    }
  }

  /*
   * Value components :
   *
   * Format :
   *   array (
   *     '[component1_key]' => array (
   *       'label' => '[component label]',
   *       'type' => '[component type]',
   *       'required' => '[booléen]',
   *       'check_data' =>  array([config LSform_rule])
   *     ),
   *     '[component2_key]' => array (
   *       'label' => 'label2',
   *       'type'  => 'select_list',
   *       'options' => array([config as LSattr_html_select_list html_options]),
   *     ),
   *     [...]
   *   )
   * Types :
   *   - 'select_list' => Component feed by a list of valeur configured like an
   *                      atribute LSattr_html :: select_list.
   *   - 'text'        => manual entry
   *
   */
  var $components = array();

 /**
  * Retourne les infos d'affichage de l'élément
  *
  * Cette méthode retourne les informations d'affichage de l'élement
  *
  * @retval array
  */
  function getDisplay(){
    $return = $this -> getLabelInfos();

    $parseValues=array();
    $invalidValues=array();
    foreach($this -> values as $val) {
      $decodedValue=json_decode($val, true);
      if (is_array($decodedValue)) {
        $parseValue=array('value' => $val);
        foreach($decodedValue as $c => $cvalue) {
          $parseValue[$c]=$this -> translateComponentValue($c,$cvalue);
        }
        $parseValues[]=$parseValue;
      }
      else {
        $invalidValues[]=$val;
      }
    }

    $return['html'] = $this -> fetchTemplate(NULL,
      array(
        'parseValues' => $parseValues,
      )
    );
    LSsession :: addCssFile('LSformElement_jsonCompositeAttribute.css');
    return $return;
  }

 /**
  * Retournne un template Smarty compilé dans le contexte d'un LSformElement
  *
  * @param[in] string $template Le template à retourner
  * @param[in] array $variables Variables Smarty à assigner avant l'affichage
  *
  * @retval string Le HTML compilé du template
  */
  function fetchTemplate($template=NULL,$variables=array()) {
    $components = $this -> components;
    foreach($components as $c => $cconf) {
      if ($cconf['type']=='select_list') {
        $components[$c]['possible_values']=$this -> getSelectListComponentPossibleValues($c);
      }
    }
    $variables['components'] = $components;

    return parent::fetchTemplate($template, $variables);
  }

  /**
   * Translate componant value
   *
   * Return an array containing :
   *  - value : untranslated value
   *  - translated : translated value
   *
   * @param[in] $c string The component name
   * @param[in] $value string The value
   *
   * @retval array
   **/
  function translateComponentValue($c,$value) {
    $retval = array (
      'translated' => $value,
      'value' => $value,
    );
    if (isset($this -> components[$c])) {
      if ($this -> components[$c]['type']=='select_list') {
        $retval['translated'] = $this -> getSelectListComponentValueLabel($c,$value);
      }
      //elseif type == 'text' => no transformation
    }
    return $retval;
  }

  /**
   * Retreive possible values of an select_list component
   *
   * @param[in] $c string The component name
   *
   * @retval array
   **/
  protected $_cache_getSelectListComponentPossibleValues=array();
  protected function getSelectListComponentPossibleValues($c) {
    if (!isset($this -> _cache_getSelectListComponentPossibleValues[$c])) {
      if (!LSsession :: loadLSclass('LSattr_html_select_list')) return;
      $this -> _cache_getSelectListComponentPossibleValues[$c]=LSattr_html_select_list :: getPossibleValues($this -> components[$c]['options'], $this -> name, $this->attr_html->attribute->ldapObject);
    }
    return $this -> _cache_getSelectListComponentPossibleValues[$c];
  }

  /**
   * Retreive value's label of an select_list component
   *
   * @param[in] $c string The component name
   * @param[in] $value string The value
   *
   * @retval array
   **/
  protected function getSelectListComponentValueLabel($c,$value) {
    if ($this -> getSelectListComponentPossibleValues($c)) {
      foreach ($this -> _cache_getSelectListComponentPossibleValues[$c] as $v => $label) {
        if (is_array($label)) {
          if (!isset($label['possible_values'])) continue;
          foreach ($label['possible_values'] as $vk => $vl)
            if ($vk == $$value) return $vl;
        }
        if ($v == $value) return $label;
      }
    }
    return;
  }

  /**
   * Retreive LSformElement value from POST data
   *
   * This method check present of this element's value in POST data and retreive
   * it to feed the array passed in paramater.
   *
   * @param[] array Reference of the array for retreived values
   *
   * @retval boolean true if value is in POST data, false instead
   */
  function getPostData(&$return) {
    if($this -> isFreeze()) {
      return true;
    }

    $count=0;
    $end=false;
    $return[$this -> name]=array();
    while ($end==false) {
      $value=array();
      $parseValue=array();
      $errors=array();
      $unemptyComponents=array();
      foreach ($this -> components as $c => $cconf) {
        if (isset($_POST[$this -> name.'__'.$c][$count])) {
          $parseValue[$c]=$_POST[$this -> name.'__'.$c][$count];
          if ($cconf['required'] && empty($parseValue[$c])) {
            $errors[]=getFData(__('Component %{c} must be defined'),__($cconf['label']));
            continue;
          }
          if (empty($parseValue[$c])) {
            continue;
          }
          $unemptyComponents[]=$c;
          if ($cconf['type']=='select_list') {
            if (!$this -> getSelectListComponentValueLabel($c, $parseValue[$c])) {
              $errors[]=getFData(__('Invalid value for component %{c}.'),__($cconf['label']));
            }
          }
          if (is_array($cconf['check_data'])) {
            foreach($cconf['check_data'] as $ruleType => $rconf) {
              $className='LSformRule_'.$ruleType;
              if (LSsession::loadLSclass($className)) {
                $r=new $className();
                if (!$r -> validate($parseValue[$c],$rconf,$this)) {
                  if (isset($rconf['msg'])) {
                    $errors[]=getFData(__($rconf['msg']),__($cconf['label']));
                  }
                  else {
                    $errors[]=getFData(__('Invalid value for component %{c}.'),__($cconf['label']));
                  }
                }
              }
              else {
                $errors[]=getFData(__("Can't validate value of component %{c}."),__($cconf['label']));
              }
            }
          }
          $value[$c]=$parseValue[$c];
        }
        else {
          // end of value break
          $end=true;
          break;
        }

      }
      if (!$end) {
        if (!empty($unemptyComponents)) {
          foreach($errors as $e) {
            $this -> form -> setElementError($this -> attr_html,$e);
          }
          $return[$this -> name][]=json_encode($value);
        }
        $count++;
      }
    }
    return true;
  }

}
